package passbaseplugin.pettersonapps.com.passbase_flutter

import android.content.Context
import android.graphics.Color
import android.view.View
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import com.passbase.passbase_sdk.Passbase
import com.passbase.passbase_sdk.PassbaseButton
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory

class PassbaseFlutterPlugin(passbase: Passbase): MethodCallHandler {
  private val passbaseRef: Passbase = passbase
  
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "passbase_flutter")
      val passbase =  Passbase(registrar.activity())
      channel.setMethodCallHandler(PassbaseFlutterPlugin(passbase))
      PassbaseButtonPlugin.registerWith(registrar, passbase, channel)
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "initialize" -> {
        val arguments = InitializeArguments(call.arguments)
        Passbase.source = 3
        passbaseRef.initialize(arguments.apiKey,
                arguments.prefillUserEmail,
                arguments.additionalArguments)

        result.success(null)
      }
      "startVerification" -> {
        passbaseRef.onCompletePassbaseVerification {
          result.success(it)
        }
        passbaseRef.onCancelPassbaseVerification {
          result.error("CANCEL", "Verification canceled", null)
        }
        passbaseRef.startVerification()
      }
      "setTestMode" -> {
        val argument = call.arguments as? Boolean
        passbaseRef.setTestMode(argument!!)
      }
      "setDarkMode" -> {
        val argument = call.arguments as? Boolean
        passbaseRef.setTheme(if (argument!!) Passbase.THEME_DARK_ON else Passbase.THEME_DARK_OFF)
      }
      else -> {
        result.notImplemented()
      }
    }
  }
}

object PassbaseButtonPlugin {
  fun registerWith(registrar: Registrar, passbase: Passbase, channel: MethodChannel) {
    val viewFactory = PassbaseButtonFactory(registrar.messenger(), passbase, channel)
    registrar.platformViewRegistry()
            .registerViewFactory("PassbaseButton", viewFactory)
  }
}

class PassbaseButtonFactory(private val messenger: BinaryMessenger, passbase: Passbase, channel: MethodChannel) :
        PlatformViewFactory(StandardMessageCodec.INSTANCE) {
  val passbase: Passbase
  val channel: MethodChannel
  
  init {
    this.passbase = passbase
    this.channel = channel
  }
  
  override fun create(context: Context,
                      id: Int,
                      args: Any?): PlatformView {
    return PassbaseButtonContainer(context,
            passbase,
            channel,
            id,
            args)
  }
}

class PassbaseButtonContainer internal constructor(private val context: Context, passbase: Passbase, channel: MethodChannel, id: Int, args: Any?) : PlatformView {
  private val passbaseButton: PassbaseButton
  private val passbaseRef: Passbase
  private val channel: MethodChannel

  init {
    this.channel = channel
    passbaseRef = passbase
    passbaseButton = PassbaseButton(context = context)
    passbaseButton.setOnClickListener { 
      passbaseRef.startVerification()
      passbaseRef.onCancelPassbaseVerification {
        channel.invokeMethod("verification_cancel", "Verification canceled")
      }
      passbaseRef.onCompletePassbaseVerification {
        channel.invokeMethod("verification_complete", it)
      }
    }
  }

  override fun dispose() {
  }
  
  override fun getView(): View {
    return passbaseButton
  }
}
